// The MIT License

// Copyright (c) 2009 Massachusetts Institute of Technology

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

import Connectable::*;
import GetPut::*;
import ClientServer::*;
import Processor::*;
import MemArb::*;
import MemTypes::*;

//AWB includes
`include "asim/provides/low_level_platform_interface.bsh"
`include "asim/provides/soft_connections.bsh"
`include "asim/provides/common_services.bsh"

// Local includes
`include "asim/provides/processor_library.bsh"
`include "asim/provides/audio_pipe_types.bsh"
`include "asim/provides/path_types.bsh"

interface PutMixerIn;
   method Action sendVoiceIn(AudioStream voiceInput);
endinterface 

interface Mixer#(Vector#(`MAX_VOICES, Volume));
   interface PutMixerIn toMixer;
   interface Get#(AudioProcessorUnit) mixerOut;
endinterface


module   [CONNECTED_MODULE]  mkMixer( Mixer#(Vector(`MAX_VOICES, Volume)));
  // Instantiate the modules
   Vector#(`MAX_VOICES, FIFO#(AudioPipeUnit)) voicesIn <- newVector();
   Vector#(`MAX_VOICES, Bool) voiceEnded <- replicate(False);

   FIFO#(AudioProcessorUnit) masterFifo <- mkFIFO();

   rule processSample  //implicit on all voiceFifos having data
   endrule
 
  // Internal connections

   interface PutMixerIn toMixer;
      method Action sendVoiceIn(AudioStream voiceInput);
	 voicesIn[voiceInput.voice].enq(voiceInput.data);
      endmethod
         
   interface Get mixerOut = fifoToGet(masterFifo);

endmodule
